# Power User Features

[TOC]

## Keyboard shortcuts

Monorail offers many keyboard shortcuts. Press `?` to see a list of shortcuts.
If your text cursor is in a text field, click outside of it or press Escape
before pressing `?`.

## Bookmarks

Every page in Monorail can be bookmarked. The URL of each page can be shared
with other users, who will see the contents of that page based on their own
permissions. Individual issue comments are also anchors that can be bookmarked.

For example, if your team uses a specific query to see the list of incoming
issues that you are responsible for triaging, copy the URL of that issue list
and share it. You may want to create a short link (a go/ link) and add a link to
it in your team documentation.

There are several query string parameters that can be included in the issue list
page links, including:

*   q= is the issue search query.

*   can= is the issue search scope number. This defaults to 2 for open issues,
    but it can be set to 1 to search all issues, or other values for other items
    in the scope menu.

*   sort= is a space-separated list of column names with the most significant
    column first. Each column name can have a leading minus-sign for descending
    sorting.

*   colspec= is a space-separated list of columns names to display.

*   groupby= is a space-separated list of columns names to group results by.

*   num= is the number of issue results to be shown on each pagination page,
    which defaults to 100.

*   mode= defaults to `list` but can be set to `chart` or `grid`.

## Deep links

It is possible to bookmark Monorail pages that display a form, such as the issue
entry page. Furthermore, you may add URL query string parameters to prefill some
form fields for the user.

On the issue entry form, these values can be filled in even if the user would
normally not see those fields offered. You can use such links in user
documentation that guides the user to report an issue that is labeled in a way
to make it show up in your team’s triage query.

Those fields are:

* `template=` the name of the issue template to use

* `summary=` initial issue summary

* `description=` initial issue description

* `labels=` a comma-separated list of labels (e.g., Type-Bug, Pri-2)

* `owner=` email address of the initial issue owner

* `status=` the status value for the new issue

* `cc=` comma-separated list of email addresses to CC

* `components=` a comma-separated list of component paths

* `blocking=` a comma-separated list of bugs the new issue will block

* `blockedon=` a comma-separated list of bugs the new issue is blocked by

## Chrome custom search engines

An easy way to skip a step when searching for issues is to define a keyword for
searching monorail in the Chrome browser. Here’s how:

1.  Pop up a context menu in the Chrome Omnibox URL field.
1.  Select "Edit Search Engines...".
1.  Click "Add".
1.  Define a short keyword and use a Monorail URL with a `%s` placeholder.

For example, you might define "mc" to search /p/chromium issues using URL
`https://bugs.chromium.org/p/chromium/issues/list?q=%s` Or, you might use a deep
link rather than a search. For example, you could start reporting a defect
summary line directly from the Omnibox by defining "md" as
`https://bugs.chromium.org/p/chromium/issues/entry?template=Defect+report+from+developer&summary=%s`

## Federated issue tracking

Monorail has some support for referencing issues in other issue tracking tools.
You can always reference any issue in any web-based issue tracking tool by
pasting the URL of that page into a Monorail issue comment. Googlers may also
want to use the "b/" shorthand for internal issues. The "b/" syntax can be used
in issue comments, blocking, blocked-on, and merged-info fields. Googlers may
see an "i" icon with an option to sign in to preview the summary lines of
referenced internal issues.

## Crbug.com shortlinks

Separate from Monorail itself, the Chromium developers maintain a service on
crbug.com that redirects requests to Monorail pages. This allows for shorter
URLs that are easier to type and fit more easily into source code comments.

The supported URLs include:

*   http://crbug.com/: Goes to the issue list in the /p/chromium project.

*   http://crbug.com/123: Goes to issue 123 in the /p/chromium project.

*   http://crbug.com/123#c4: Goes to issue chromium:123 comment 4.

*   http://crbug.com/new or http://crbug.new: Goes to issue entry form in the
    /p/chromium project.

*   http://crbug.com/PROJECT/: Goes to the issue list in the specified project.

*   http://crbug.com/PROJECT/123: Goes to issue 123 in the specified project.

*   http://crbug.com/PROJECT/new: Goes to the issue entry form in the specified
    project.

## Autolinking

When you type an issue description or comment, Monorail will automatically
convert some parts of the text into HTML links. The following are supported:

*   `issue 123` or `bug 123` link to issue 123 in the current project.

*   `issue project:123` links to issue 123 in the specified project.

*   `b/123` links to issue 123 in Google’s internal issue tracker.

*   `cl/123` links to changelist 123 in Google’s internal version control
    system.

*   A git hash links to http://crrev.com for the specified commit, unless
    otherwise specified in the project.

*   `comment 6` or `#c6` link to the 6th comment in the current issue thread.

*   `go/link-name` links to your page through Google’s shortlink service

## Commit-log message integration

Most projects are configured to allow the Git Watcher tool to post comments to
issues. Including `Bug: <bug_id...>` in a commit message triggers Git Watcher to
post a comment mentioning the commit on the issue when the commit's CL is
submitted. Including `Fixed: <bug_id...>` in a commit message triggers Git
Watcher to set the referenced bugs' status to "Fixed".

Both the `Bug:` and the `Fixed:` automations accept bug IDs in multiple formats:
*   `Fixed: n`
*   `Fixed: project:n`
*   `Fixed: <Issue Tracker URL>`
